/* This is the console executable, that makes use of the BullCow class.
This acts as the view in a MVC pattern, and is responsible for all
user interaction. For game logic please see FBullCowGame class.
*/

#pragma once
#include <iostream>
#include <string>
#include <sstream>
#include "FBullCowGame.h"

//to make syntax Unreal-friendly
using Ftext = std::string;
using int32 = int;

//enable user to choose custom word length
int32 MaxWordLength;
Ftext MaxWordLengthProxy;

//function prototypes as outside a class
void PrintIntro();
void SetGameDifficulty();
void PrintInstructions();
void PlayGame();
void PrintGameSummary();
bool AsktoPlayAgain();
Ftext GetValidGuess();

FBullCowGame BCGame; //Instantiate a new game, which we re-use across plays

int main()
{
	bool bPlayAgain = true;
	do {
		BCGame.ResetGame();
		PrintIntro();
		SetGameDifficulty();
		BCGame.SetHiddenWord();
		PrintInstructions();
		PlayGame();
		PrintGameSummary();
		bPlayAgain = AsktoPlayAgain();
	} while (bPlayAgain);
	return 0;
}

void PrintIntro() {
	std::cout << "Welcome to Bulls and Cows, a fun word game\n";
	std::cout << "      ^     ^         _     _\n";
	std::cout << "     /_\\___/_\\       ( )___( )\n";
	std::cout << "    |         |      |       | \n";
	std::cout << "    | ^     ^ |      | _   _ |\n";
	std::cout << "    | O     O |      | U   U |\n";
	std::cout << "    | ^     ^ |      |       |\n";
	std::cout << "    |   . .   |      |  n n  |\n";
	std::cout << "    |       o |      |_____u_|\n";
	std::cout << "    \\_________/                \n";
	std::cout << "        BULL            COW    \n\n";
	return;
}

void SetGameDifficulty()
{
	//enable user to select the word length and associated difficulty of the game
	std::cout << "Choose a word length to set the game's difficulty. " << BCGame.GetEasiestDiffficulty() << " is the easiest and ";
	std::cout << BCGame.GetHardestDiffficulty() << " is the hardest! ";
	do {
		getline(std::cin,MaxWordLengthProxy);
		std::stringstream Proxy(MaxWordLengthProxy);
		MaxWordLength = 0;
		Proxy >> MaxWordLength;
		if (MaxWordLength <= BCGame.GetHardestDiffficulty() && MaxWordLength >= BCGame.GetEasiestDiffficulty() && !std::cin.fail())
		{
			std::cout << "Hit \"Enter\" to launch the game!";
			std::cin.clear();
			std::cin.ignore();
			break;
		}
		else
		{
			std::cout << "\nPlease enter a numeric value between " << BCGame.GetEasiestDiffficulty() << " and " << BCGame.GetHardestDiffficulty() << ". ";
		}
	} while (!std::cin.fail() || MaxWordLength > BCGame.GetHardestDiffficulty() || MaxWordLength < BCGame.GetEasiestDiffficulty());
	std::cout << std::endl;
	return;
}

void PrintInstructions()
{
	// introduce the game to the user
	std::cout << "Instructions for how to play:\n";
	std::cout << "\nEach time you guess a word, the number of \"Bulls\" and \"Cows\" will show up below your word. ";
	std::cout << "\"Bulls\" indicate the number of correct letters in the correct positions whereas \"Cows\" indicate the number of correct letters in the wrong location. ";
	std::cout << "The objective of the game is to guess the hidden word correctly and get the highest number of \"Bulls\"  possible. ";
	std::cout << "Bear in mind that each of your guesses has to be an isogram (a word with no repeating letters).\n";
	std::cout << "\nCan you guess the " << BCGame.GetHiddenWordLength();
	std::cout << " letter isogram in " << BCGame.GetMaxTries();
	std::cout << " tries? Let's find out!\n";
	return;
}

void PrintGameSummary()
{
	// let the user know if they have won or lost the game
	if(BCGame.IsGameWon())
	{
		std::cout << "\nYou won the game! Congratulations!";
	}
	else
	{
		std::cout << "\nBetter luck next time...";
	}
	return;
}

//plays a single game through to completion
void PlayGame()
{	int32 MyMaxTries = BCGame.GetMaxTries();
	
	//loops for the number of guesses while the game
	// is NOT won and there are still tries remaining
	while (!BCGame.IsGameWon() && BCGame.GetCurrentTry()<=MyMaxTries)
	{	Ftext Guess = GetValidGuess();
			
	//submit valid guess to the game and receive counts
	FBullCowCount BullCowCount = BCGame.SubmitValidGuess(Guess);
	std::cout << "Bulls = " << BullCowCount.Bulls;
	std::cout << ". Cows = " << BullCowCount.Cows;
	
	std::cout << std::endl;
 	}
	return;
}

bool AsktoPlayAgain()
{
	std::cout << std::endl;
	std::cout << "Do you want to play again? ";
	Ftext Response = "";
	getline(std::cin, Response);
	std::cout << std::endl;
	return (Response[0] == 'Y') || (Response[0] == 'y');
}


//loop continually until the user gets a valid guess
Ftext GetValidGuess()
{	
	Ftext Guess = "";
	EGuessStatus Status = EGuessStatus::Invalid_status;
	do	{	//ask the user for his or her guess
		int32 MyCurrentTry = BCGame.GetCurrentTry();
		int32 MaxTries = BCGame.GetMaxTries();
		std::cout << std::endl << "Try " << MyCurrentTry << " of " << MaxTries << "\n";
		std::cout << "Provide your best guess! ";
		getline(std::cin, Guess);
		Status = BCGame.ValidateAnswer(Guess);
		switch (Status)
		{
		case EGuessStatus::Word_length:
			std::cout << "Please enter a " << BCGame.GetHiddenWordLength() << " letter word.\n";
			break;

		case EGuessStatus::Not_Isogram:
			std::cout << "Please enter an isogram.\n";
			break;

		case EGuessStatus::Not_lowercase:
			std::cout << "Please your word in lowercase letters only.\n";
			break;

		default:
			break; //assume guess is valid
		}
	} while (Status != EGuessStatus::Okay); //keep looping until we get no errors
	return Guess;
}
